package com.xzzn.ems.service.impl;

import java.util.List;
import com.xzzn.common.utils.DateUtils;
import com.xzzn.ems.domain.EmsPcsData;
import com.xzzn.ems.mapper.EmsPcsDataMapper;
import com.xzzn.ems.service.IEmsPcsDataService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

/**
 * PCS数据Service业务层处理
 *
 * @author xzzn
 * @date 2025-06-27
 */
@Service
public class EmsPcsDataServiceImpl implements IEmsPcsDataService
{
    @Autowired
    private EmsPcsDataMapper emsPcsDataMapper;

    /**
     * 查询PCS数据
     *
     * @param id PCS数据主键
     * @return PCS数据
     */
    @Override
    public EmsPcsData selectEmsPcsDataById(Long id)
    {
        return emsPcsDataMapper.selectEmsPcsDataById(id);
    }

    /**
     * 查询PCS数据列表
     *
     * @param emsPcsData PCS数据
     * @return PCS数据
     */
    @Override
    public List<EmsPcsData> selectEmsPcsDataList(EmsPcsData emsPcsData)
    {
        return emsPcsDataMapper.selectEmsPcsDataList(emsPcsData);
    }

    /**
     * 新增PCS数据
     *
     * @param emsPcsData PCS数据
     * @return 结果
     */
    @Override
    public int insertEmsPcsData(EmsPcsData emsPcsData)
    {
        emsPcsData.setCreateTime(DateUtils.getNowDate());
        return emsPcsDataMapper.insertEmsPcsData(emsPcsData);
    }

    /**
     * 修改PCS数据
     *
     * @param emsPcsData PCS数据
     * @return 结果
     */
    @Override
    public int updateEmsPcsData(EmsPcsData emsPcsData)
    {
        emsPcsData.setUpdateTime(DateUtils.getNowDate());
        return emsPcsDataMapper.updateEmsPcsData(emsPcsData);
    }

    /**
     * 批量删除PCS数据
     *
     * @param ids 需要删除的PCS数据主键
     * @return 结果
     */
    @Override
    public int deleteEmsPcsDataByIds(Long[] ids)
    {
        return emsPcsDataMapper.deleteEmsPcsDataByIds(ids);
    }

    /**
     * 删除PCS数据信息
     *
     * @param id PCS数据主键
     * @return 结果
     */
    @Override
    public int deleteEmsPcsDataById(Long id)
    {
        return emsPcsDataMapper.deleteEmsPcsDataById(id);
    }
}
