package com.xzzn.ems.service.impl;

import com.xzzn.common.utils.DateUtils;
import com.xzzn.ems.domain.vo.DateSearchRequest;
import com.xzzn.ems.domain.vo.ElectricDataInfoVo;
import com.xzzn.ems.domain.vo.SiteMonitorDataVo;
import com.xzzn.ems.mapper.EmsPcsDataMapper;
import com.xzzn.ems.service.IEmsStatsReportService;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.time.LocalDate;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * 统计报表数据Service业务层处理
 *
 * @author xzzn
 * @date 2025-06-29
 */
@Service
public class EmsStatsReportServiceImpl implements IEmsStatsReportService
{

    private static final Logger log = LoggerFactory.getLogger(EmsStatsReportServiceImpl.class);

    @Autowired
    private EmsPcsDataMapper emsPcsDataMapper;

    @Override
    public ElectricDataInfoVo getElectricDataResult(DateSearchRequest requestVo) {
        ElectricDataInfoVo electricDataInfoVo = new ElectricDataInfoVo();

        Date startDate = requestVo.getStartDate();
        Date endDate = requestVo.getEndDate();
        if (startDate == null && endDate == null) {
            // 如果没有传时间，默认从今天往前7天
            LocalDate sevenDaysAgo = LocalDate.now().minusDays(6);
            startDate = DateUtils.toDate(sevenDaysAgo);
            endDate = new Date();
        }

        List<SiteMonitorDataVo> dataList = new ArrayList();
        //  开始日期和结束日期同一天，展示 0-24 小时数据
        if (DateUtils.isSameDay(startDate, endDate)){
            electricDataInfoVo.setUnit("时");
            endDate = DateUtils.addDays(endDate, 1);
            dataList = emsPcsDataMapper.getPcsDataByHour(requestVo.getSiteId(), startDate, endDate);
        } else if (DateUtils.differentDaysByMillisecond(endDate, startDate) >= 1
                && DateUtils.differentDaysByMillisecond(endDate, startDate) < 30){
            electricDataInfoVo.setUnit("日");
            endDate = DateUtils.addDays(endDate, 1);
            //  开始日期-结束日期大于 1 天，小于30 天，按天展示数据
            dataList = emsPcsDataMapper.getPcsDataByDay(requestVo.getSiteId(), startDate, endDate);
        } else if (DateUtils.differentDaysByMillisecond(endDate, startDate) >= 30){
            electricDataInfoVo.setUnit("月");
            endDate = DateUtils.addDays(endDate, 1);
            //  开始日期-结束日期大于 1 个月，按月展示数据
            dataList = emsPcsDataMapper.getPcsDataByMonth(requestVo.getSiteId(), startDate, endDate);
        }
        //  根据时间获取每天的充放电量
        if (!CollectionUtils.isEmpty(dataList)){
            // 总充、总放、效率
            BigDecimal totalDischarge = new BigDecimal(0);
            BigDecimal totalCharge = new BigDecimal(0);
            BigDecimal efficiency = new BigDecimal(0);

            for (SiteMonitorDataVo siteMonitorDataVo : dataList) {
                totalDischarge = totalDischarge.add(siteMonitorDataVo.getDisChargedCap() == null ? BigDecimal.ZERO : siteMonitorDataVo.getDisChargedCap());
                totalCharge = totalCharge.add(siteMonitorDataVo.getChargedCap() ==  null ? BigDecimal.ZERO : siteMonitorDataVo.getChargedCap());

                // 计算单天的效率
                BigDecimal dailyEfficiency = new BigDecimal(0);
                if ( siteMonitorDataVo.getDisChargedCap().compareTo(BigDecimal.ZERO)>0){
                    dailyEfficiency = siteMonitorDataVo.getDisChargedCap().divide(siteMonitorDataVo.getChargedCap(), 2, RoundingMode.HALF_UP);
                }
                siteMonitorDataVo.setDailyEfficiency(dailyEfficiency);
            }
            if ( totalCharge.compareTo(BigDecimal.ZERO)>0){
                efficiency = totalDischarge.divide(totalCharge, 2, RoundingMode.HALF_UP);
            }
            electricDataInfoVo.setSevenDayDisChargeStats(dataList);
            electricDataInfoVo.setTotalDisChargedCap(totalDischarge);
            electricDataInfoVo.setTotalChargedCap(totalCharge);
            electricDataInfoVo.setEfficiency(efficiency);
        }
        return electricDataInfoVo;
    }
}
