package com.xzzn.ems.service.impl;

import com.xzzn.common.constant.RedisKeyConstants;
import com.xzzn.common.core.redis.RedisCache;
import com.xzzn.common.enums.AmmeterCategory;
import com.xzzn.common.enums.DeviceCategory;
import com.xzzn.common.utils.DateUtils;
import com.xzzn.common.utils.StringUtils;
import com.xzzn.ems.domain.*;
import com.xzzn.ems.domain.vo.*;
import com.xzzn.ems.mapper.*;
import com.xzzn.ems.service.ISingleSiteService;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.math.BigDecimal;
import java.util.*;

/**
 * 单点监控 服务层实现
 */
@Service
public class SingleSiteServiceImpl implements ISingleSiteService {

    private static final String CLUSTER_DATA_TEP = "温度";

    private static final String CLUSTER_DATA_VOLTAGE = "电压";

    private static final String CLUSTER_DATA_SOC = "SOC";

    private static final String AMMETER_DEVICE_LOAD = "LOAD";

    private static final String AMMETER_DEVICE_METE = "METE";

    @Autowired
    private EmsPcsDataMapper emsPcsDataMapper;
    @Autowired
    private EmsAlarmRecordsMapper emsAlarmRecordsMapper;
    @Autowired
    private EmsBatteryDataMapper emsBatteryDataMapper;
    @Autowired
    private EmsBatteryStackMapper emsBatteryStackMapper;
    @Autowired
    private EmsBatteryClusterMapper emsBatteryClusterMapper;
    @Autowired
    private EmsPcsBranchDataMapper emsPcsBranchDataMapper;
    @Autowired
    private EmsCoolingDataMapper emsCoolingDataMapper;
    @Autowired
    private EmsAmmeterDataMapper emsAmmeterDataMapper;
    @Autowired
    private EmsDevicesSettingMapper emsDevicesSettingMapper;

    @Autowired
    private RedisCache redisCache;

    @Override
    public SiteMonitorHomeVo getSiteMonitorDataVo(String siteId) {
        SiteMonitorHomeVo siteMonitorHomeVo = new SiteMonitorHomeVo();

        if (!StringUtils.isEmpty(siteId)) {
            // 获取单站点的总充+总放+电网实时功率
            Map<String,BigDecimal> pcsMap  = emsPcsDataMapper.getPcsTotalChargeData(siteId);
            if (pcsMap != null) {
                siteMonitorHomeVo.setTotalChargedCap(pcsMap.get("totalChargedCap"));
                siteMonitorHomeVo.setTotalDischargedCap(pcsMap.get("totalDischargedCap"));
                siteMonitorHomeVo.setGridNrtPower(pcsMap.get("gridNrtPower"));
            }
            // 实时告警数据 名称+状态+告警内容
            List<SiteMonitorHomeAlarmVo> siteMonitorHomeAlarmVo = emsAlarmRecordsMapper.getAlarmRecordsBySiteId(siteId);
            siteMonitorHomeVo.setSiteMonitorHomeAlarmVo(siteMonitorHomeAlarmVo);
            // 能量数据
            List<SiteMonitorDataVo> siteMonitorDataVoList = emsPcsDataMapper.getPcsDataBySiteId(siteId,6);
            if (!CollectionUtils.isEmpty(siteMonitorDataVoList)) {
                for (SiteMonitorDataVo sitePcsData : siteMonitorDataVoList) {
                    // 获取当天的充电量+放电量
                    String pcsDate = sitePcsData.getAmmeterDate();
                    boolean isToday= checkIsToday(pcsDate);
                    if(isToday){
                        siteMonitorHomeVo.setDayChargedCap(sitePcsData.getChargedCap());
                        siteMonitorHomeVo.setDayDisChargedCap(sitePcsData.getDisChargedCap());
                    }
                }
                // 储能可用电量
                BigDecimal energyStorageAvailElec = siteMonitorHomeVo.getTotalDischargedCap().subtract(siteMonitorHomeVo.getTotalChargedCap());
                siteMonitorHomeVo.setEnergyStorageAvailElec(energyStorageAvailElec);
            }
            siteMonitorHomeVo.setSiteMonitorDataVo(siteMonitorDataVoList);
        }

        return siteMonitorHomeVo;
    }

    private boolean checkIsToday(String pcsDate) {
        boolean flag = false;
        Calendar calendar = Calendar.getInstance();
        int todayMonth = calendar.get(Calendar.MONTH) + 1;
        int todayDay = calendar.get(Calendar.DAY_OF_MONTH);
        if (StringUtils.isNotEmpty(pcsDate)){
            String[] pcsDateArray = pcsDate.split("/");
            if (todayMonth == Integer.parseInt(pcsDateArray[0]) &&
                    todayDay == Integer.parseInt(pcsDateArray[1])) {
                flag = true;
            }
        }
        return flag;
    }

    // 获取单站监控实时运行头部数据
    @Override
    public SiteMonitorRunningHeadInfoVo getSiteRunningHeadInfo(String siteId) {
        SiteMonitorRunningHeadInfoVo siteMonitorRunningHeadInfoVo = new SiteMonitorRunningHeadInfoVo();

        if (!StringUtils.isEmpty(siteId)) {
            // 实时有功功率/实时无功功率/今日充电量/今日放电量
            siteMonitorRunningHeadInfoVo = emsPcsDataMapper.getSiteRunningHeadInfo(siteId);
            // 电池簇SOC
            // 电池堆SOH
            EmsBatteryData emsBatteryData = emsBatteryDataMapper.getBatteryDataBySiteId(siteId);
            if (emsBatteryData != null) {
                siteMonitorRunningHeadInfoVo.setSoc(emsBatteryData.getSoc());
                siteMonitorRunningHeadInfoVo.setSoh(emsBatteryData.getSoh());
            }
        }

        return siteMonitorRunningHeadInfoVo;
    }

    //  获取单站监控实时运行-储能功率数据
    @Override
    public SiteMonitorRuningInfoVo getRunningGraphStorage(String siteId) {
        SiteMonitorRuningInfoVo siteMonitorRuningInfoVo = new SiteMonitorRuningInfoVo();
        if (!StringUtils.isEmpty(siteId)) {
            // 时间暂定今日+昨日
            Date today = new Date();
            Date yesterday = DateUtils.addDays(today, -1);
            //储能功率list
            List<EnergyStoragePowVo> energyStoragePowList = emsPcsDataMapper.getStoragePowerList(siteId, yesterday, today);
            siteMonitorRuningInfoVo.setEnergyStoragePowList(energyStoragePowList);
        }
        return siteMonitorRuningInfoVo;
    }
    //  获取单站监控实时运行-poc平均温度
    @Override
    public SiteMonitorRuningInfoVo getRunningGraphPcsTemp(String siteId) {
        SiteMonitorRuningInfoVo siteMonitorRuningInfoVo = new SiteMonitorRuningInfoVo();
        if (!StringUtils.isEmpty(siteId)) {
            // 时间暂定今日+昨日
            Date today = new Date();
            Date yesterday = DateUtils.addDays(today, -1);
            //pcs平均温度list
            List<PCSAveTempVo> pcsAveTempVos = emsPcsDataMapper.getPcsAveTempList(siteId, yesterday, today);
            siteMonitorRuningInfoVo.setPcsAveTempList(pcsAveTempVos);
        }
        return siteMonitorRuningInfoVo;
    }
    //  获取单站监控实时运行-电池平均soc
    @Override
    public SiteMonitorRuningInfoVo getRunningGraphBatterySoc(String siteId) {
        SiteMonitorRuningInfoVo siteMonitorRuningInfoVo = new SiteMonitorRuningInfoVo();
        if (!StringUtils.isEmpty(siteId)) {
            // 时间暂定今日+昨日
            Date today = new Date();
            Date yesterday = DateUtils.addDays(today, -1);
            //电池平均soclist
            List<BatteryAveSOCVo> batteryAveSOCList = emsPcsDataMapper.getAveSocList(siteId, yesterday, today);
            siteMonitorRuningInfoVo.setBatteryAveSOCList(batteryAveSOCList);
        }
        return siteMonitorRuningInfoVo;
    }
    //  获取单站监控实时运行-电池平均温度
    @Override
    public SiteMonitorRuningInfoVo getRunningGraphBatteryTemp(String siteId) {
        SiteMonitorRuningInfoVo siteMonitorRuningInfoVo = new SiteMonitorRuningInfoVo();
        if (!StringUtils.isEmpty(siteId)) {
            // 时间暂定今日+昨日
            Date today = new Date();
            Date yesterday = DateUtils.addDays(today, -1);
            //电池平均温度list
            List<BatteryAveTempVo> batteryAveTempList = emsPcsDataMapper.getBatteryAveTempList(siteId, yesterday, today);
            siteMonitorRuningInfoVo.setBatteryAveTempList(batteryAveTempList);
        }
        return siteMonitorRuningInfoVo;
    }

    // 根据site_id获取pcs详细数据+支路数据
    @Override
    public List<PcsDetailInfoVo> getPcsDetailInfo(String siteId) {
        List<PcsDetailInfoVo> pcsDetailInfoVoList = new ArrayList<>();

        if (!StringUtils.isEmpty(siteId)) {
            // 获取该设备下所有pcs的id
            List<Map<String, Object>> pcsIds = emsDevicesSettingMapper.getDeviceInfosBySiteIdAndCategory(siteId, DeviceCategory.PCS.getCode());

            for (Map<String, Object> pcsDevice : pcsIds) {
                PcsDetailInfoVo pcsDetailInfoVo = new PcsDetailInfoVo();
                pcsDetailInfoVo.setDeviceName(pcsDevice.get("deviceName").toString());
                pcsDetailInfoVo.setCommunicationStatus(pcsDevice.get("communicationStatus").toString());
                // 从redis取pcs单个详细数据
                String pcsId = pcsDevice.get("id").toString();
                EmsPcsData pcsData = redisCache.getCacheObject(RedisKeyConstants.PCS +siteId+"_"+pcsId);
                if  (pcsData != null) {
                    BeanUtils.copyProperties(pcsData, pcsDetailInfoVo);
                }
                // 支路信息数据
                List<PcsBranchInfo> pcsBranchInfoList  = new  ArrayList<>();
                processBranchDataInfo(siteId,pcsId,pcsBranchInfoList);
                pcsDetailInfoVo.setPcsBranchInfoList(pcsBranchInfoList);

                pcsDetailInfoVoList.add(pcsDetailInfoVo);
            }
        }
        return pcsDetailInfoVoList;
    }

    private void processBranchDataInfo(String siteId, String pcsId, List<PcsBranchInfo> pcsBranchInfoList) {
        if (!StringUtils.isEmpty(pcsId)) {
            List<EmsPcsBranchData> pcsBranchData = redisCache.getCacheObject(RedisKeyConstants.BRANCH +siteId+"_"+pcsId);
            if (pcsBranchData != null) {
                for(EmsPcsBranchData emsPcsBranchData : pcsBranchData) {
                    PcsBranchInfo pcsBranchInfo = new PcsBranchInfo();
                    BeanUtils.copyProperties(emsPcsBranchData, pcsBranchInfo);
                    pcsBranchInfoList.add(pcsBranchInfo);
                }
            }
        }
    }

    // 获取BMS总览数据
    @Override
    public List<BMSOverViewVo> getBMSOverView(String siteId) {
        List<BMSOverViewVo> bmsOverViewVoList = new ArrayList<>();

        // 获取所有电池堆
        if (!StringUtils.isEmpty(siteId)) {
            List<Map<String, Object>> stackIds = emsDevicesSettingMapper.getDeviceInfosBySiteIdAndCategory(siteId, DeviceCategory.STACK.getCode());
            for (Map<String, Object> stackDevice : stackIds) {
                BMSOverViewVo bmsOverViewVo = new BMSOverViewVo();
                bmsOverViewVo.setDeviceName(stackDevice.get("deviceName").toString());
                // 从redis取堆单个详细数据
                String stackId = stackDevice.get("id").toString();
                EmsBatteryStack stackData = redisCache.getCacheObject(RedisKeyConstants.STACK +siteId+"_"+stackId);
                if  (stackData != null) {
                    BeanUtils.copyProperties(stackData, bmsOverViewVo);
                }
                // 下面簇列表数据
                if (!StringUtils.isEmpty(stackId)) {
                    getBMSClusterListInfo(siteId,stackId,bmsOverViewVo);
                }
                bmsOverViewVoList.add(bmsOverViewVo);
            }
        }
        return bmsOverViewVoList;
    }

    private void getBMSClusterListInfo(String siteId, String stackId, BMSOverViewVo bmsOverViewVo) {
        List<BMSBatteryDataList> batteryDataList = new ArrayList();

        List<Map<String, Object>> clusterIds = emsDevicesSettingMapper.getDeviceInfoByParentId(stackId);
        for (Map<String, Object> clusterDevice : clusterIds) {
            BMSBatteryDataList bmsBatteryDataList= new BMSBatteryDataList();
            // 从redis取单个簇的详细数据
            String clusterId = clusterDevice.get("id").toString();
            bmsBatteryDataList.setClusterId(clusterId);
            EmsBatteryCluster clusterData = redisCache.getCacheObject(RedisKeyConstants.CLUSTER + siteId + "_" + clusterId);
            if (clusterData != null) {
                BeanUtils.copyProperties(clusterData, bmsBatteryDataList);
            }
            batteryDataList.add(bmsBatteryDataList);
        }

        bmsOverViewVo.setBatteryDataList(batteryDataList);
    }

    // 获取BMS电池簇数据
    @Override
    public List<BMSBatteryClusterVo> getBMSBatteryCluster(String siteId) {
        List<BMSBatteryClusterVo> bmsBatteryClusterVoList = new ArrayList<>();

        if (!StringUtils.isEmpty(siteId)) {
            // 获取所有设备下的电池簇id
            List<Map<String, Object>> clusterIds = emsDevicesSettingMapper.getDeviceInfosBySiteIdAndCategory(siteId, DeviceCategory.CLUSTER.getCode());
            for (Map<String, Object> clusterDevice : clusterIds) {
                BMSBatteryClusterVo bmsBatteryClusterVo = new BMSBatteryClusterVo();
                bmsBatteryClusterVo.setDeviceName(clusterDevice.get("deviceName").toString());
                // 从redis取单个簇详细数据
                String clusterId = clusterDevice.get("id").toString();
                EmsBatteryCluster clusterData = redisCache.getCacheObject(RedisKeyConstants.CLUSTER +siteId+"_"+clusterId);
                if  (clusterData != null) {
                    BeanUtils.copyProperties(clusterData, bmsBatteryClusterVo);
                    // 处理单体电池数据-平均/最大/最小
                    List<BMSBatteryClusterDataList> clusterDataList = new ArrayList<>();
                    dealWithBatteryClusterData(clusterData,clusterDataList);
                    bmsBatteryClusterVo.setBatteryDataList(clusterDataList);
                }

                bmsBatteryClusterVoList.add(bmsBatteryClusterVo);
            }
        }
        return bmsBatteryClusterVoList;
    }

    private void dealWithBatteryClusterData(EmsBatteryCluster clusterData, List<BMSBatteryClusterDataList> clusterDataList) {
        BMSBatteryClusterDataList voltageData = new BMSBatteryClusterDataList();
        BMSBatteryClusterDataList tempData = new BMSBatteryClusterDataList();
        BMSBatteryClusterDataList socData = new BMSBatteryClusterDataList();

        // 设值
        voltageData.setDataName(CLUSTER_DATA_VOLTAGE);
        voltageData.setAvgData(clusterData.getAvgCellVoltage());
        voltageData.setMaxData(clusterData.getMaxCellVoltage());
        voltageData.setMaxDataID(clusterData.getMaxCellVoltageId().toString());
        voltageData.setMinData(clusterData.getMinCellVoltage());
        voltageData.setMinDataID(clusterData.getMinCellVoltageId().toString());
        tempData.setDataName(CLUSTER_DATA_TEP);
        tempData.setAvgData(clusterData.getAvgCellTemp());
        tempData.setMaxData(clusterData.getMaxCellTemp());
        tempData.setMaxDataID(clusterData.getMaxCellTempId().toString());
        tempData.setMinData(clusterData.getMinCellTemp());
        tempData.setMinDataID(clusterData.getMinCellTempId().toString());
        socData.setDataName(CLUSTER_DATA_SOC);
        socData.setAvgData(clusterData.getCurrentSoc());
        socData.setMaxData(clusterData.getMaxCellSoc());
        socData.setMaxDataID(clusterData.getMaxCellSocId().toString());
        socData.setMinData(clusterData.getMinCellSoc());
        socData.setMinDataID(clusterData.getMinCellSocId().toString());

        clusterDataList.add(voltageData);
        clusterDataList.add(tempData);
        clusterDataList.add(socData);
    }

    // 获取液冷设备参数
    @Override
    public  List<EmsCoolingData> getCoolingDataList(String siteId) {
        List<EmsCoolingData> emsCoolingDataList = new ArrayList<>();
        if (!StringUtils.isEmpty(siteId)) {
            emsCoolingDataList = emsCoolingDataMapper.getCoolingDataList(siteId);
        }
        return emsCoolingDataList;
    }


    /**
     * 根据电池堆获取单体电池数据
     * @param clusterDeviceId
     * @return
     */
    @Override
    public  List<BatteryDataStatsListVo> getClusterDataInfoList(String clusterDeviceId,String  siteId) {
        List<BatteryDataStatsListVo> batteryDataStatsListVo = new ArrayList<>();
        List<Map<String, Object>> clusterIds = new ArrayList<>();
        if (StringUtils.isEmpty(clusterDeviceId)) {
            clusterIds = emsDevicesSettingMapper.getDeviceInfosBySiteIdAndCategory(siteId, DeviceCategory.CLUSTER.getCode());
        } else {
            Map<String, Object> map = new HashMap<>();
            map.put("id", clusterDeviceId);
            clusterIds.add(map);
        }
        for (Map<String, Object> clusterDevice : clusterIds) {
            // 从redis取单个簇详细数据
            String clusterId = clusterDevice.get("id").toString();
            List<EmsBatteryData> batteryDataList = redisCache.getCacheList(RedisKeyConstants.BATTERY + siteId + "_" + clusterId);
            if (batteryDataList != null) {
               for (EmsBatteryData batteryData : batteryDataList) {
                   BatteryDataStatsListVo batteryDataStatsVo = new BatteryDataStatsListVo();
                   BeanUtils.copyProperties(batteryData, batteryDataStatsVo);
                   batteryDataStatsListVo.add(batteryDataStatsVo);
               }
            }
        }
        return batteryDataStatsListVo;
    }

    /**
     * 电表数据信息
     * @param siteId
     * @return
     */
    @Override
    public AmmeterDataResponse getAmmeterDataList(String siteId) {
        AmmeterDataResponse ammeterResponse = new AmmeterDataResponse();

        if (!StringUtils.isEmpty(siteId)) {
            // 先获取电表设备
            List<Map<String, Object>> ammeterIdList = emsDevicesSettingMapper.getDeviceInfosBySiteIdAndCategory(siteId, DeviceCategory.AMMETER.getCode());

            for (Map<String, Object> ammeterDevice : ammeterIdList) {
                String ammeterId = ammeterDevice.get("id").toString();
                // 从redis取总表详细数据
                EmsAmmeterData ammeterData = redisCache.getCacheObject(RedisKeyConstants.AMMETER + siteId + "_" +ammeterId);
                // 判断电表类型
                if (AMMETER_DEVICE_LOAD.equals(ammeterId)) {
                    AmmeterLoadDataVo ammeterLoadDataVo = new AmmeterLoadDataVo();
                    ammeterLoadDataVo.setDeviceName(ammeterDevice.get("deviceName").toString());
                    ammeterLoadDataVo.setEmsCommunicationStatus(ammeterDevice.get("communicationStatus").toString());
                    // 处理总表数据
                    dealAmmeterLoadData(ammeterData,ammeterLoadDataVo);
                    ammeterResponse.setAmmeterLoadDataVoList(ammeterLoadDataVo);
                } else if (AMMETER_DEVICE_METE.equals(ammeterId)) {
                    AmmeterMeteDataVo ammeterMeteDataVo = new AmmeterMeteDataVo();
                    ammeterMeteDataVo.setDeviceName(ammeterDevice.get("deviceName").toString());
                    ammeterMeteDataVo.setEmsCommunicationStatus(ammeterDevice.get("communicationStatus").toString());
                    // 处理储能表数据
                    dealAmmeterMeteData(ammeterData,ammeterMeteDataVo);
                    ammeterResponse.setAmmeterMeteDataVoList(ammeterMeteDataVo);
                }
            }
        }
        return ammeterResponse;
    }

    private void dealAmmeterMeteData(EmsAmmeterData ammeterData, AmmeterMeteDataVo ammeterMeteDataVo) {
        if  (ammeterData != null) {
            // 数据更新时间
            ammeterMeteDataVo.setDataUpdateTime(ammeterData.getDataUpdateTime());

            List<MeteDataDetailInfo> meteDataDetailInfos = new ArrayList<>();
            // 拼接数据
            // a相
            MeteDataDetailInfo meteDataDetailInfo1 = new MeteDataDetailInfo();
            meteDataDetailInfo1.setCategory(AmmeterCategory.A_POWER.getInfo());
            meteDataDetailInfo1.setActivePower(ammeterData.getPhaseAActivePower());
            meteDataDetailInfo1.setReactivePower(ammeterData.getPhaseAReactivePower());
            meteDataDetailInfos.add(meteDataDetailInfo1);
            // b相
            MeteDataDetailInfo meteDataDetailInfo2 = new MeteDataDetailInfo();
            meteDataDetailInfo2.setCategory(AmmeterCategory.B_POWER.getInfo());
            meteDataDetailInfo2.setActivePower(ammeterData.getPhaseBActivePower());
            meteDataDetailInfo2.setReactivePower(ammeterData.getPhaseBReactivePower());
            meteDataDetailInfos.add(meteDataDetailInfo2);
            // c相
            MeteDataDetailInfo meteDataDetailInfo3 = new MeteDataDetailInfo();
            meteDataDetailInfo3.setCategory(AmmeterCategory.C_POWER.getInfo());
            meteDataDetailInfo3.setActivePower(ammeterData.getPhaseCActivePower());
            meteDataDetailInfo3.setReactivePower(ammeterData.getPhaseCReactivePower());
            meteDataDetailInfos.add(meteDataDetailInfo3);
            ammeterMeteDataVo.setMeteDataDetailInfo(meteDataDetailInfos);
        }
    }

    private void dealAmmeterLoadData(EmsAmmeterData ammeterData, AmmeterLoadDataVo ammeterDataVo) {
        if  (ammeterData != null) {
            // 数据更新时间
            ammeterDataVo.setDataUpdateTime(ammeterData.getDataUpdateTime());

            List<LoadDataDetailInfo> loadDataDetailInfos = new ArrayList<>();
            // 拼接数据
            // 组合有功
            LoadDataDetailInfo ammeterDataDetailInfo1 = new LoadDataDetailInfo();
            ammeterDataDetailInfo1.setCategory(AmmeterCategory.CURRENT_COMB_ACTIVE.getInfo());
            ammeterDataDetailInfo1.setTotalKwh(ammeterData.getCurrentCombActiveTotal());
            ammeterDataDetailInfo1.setPeakKwh(ammeterData.getCurrentCombActivePeak());
            ammeterDataDetailInfo1.setHighKwh(ammeterData.getCurrentCombActiveHigh());
            ammeterDataDetailInfo1.setFlatKwh(ammeterData.getCurrentCombActiveFlat());
            ammeterDataDetailInfo1.setValleyKwh(ammeterData.getCurrentCombActiveValley());
            loadDataDetailInfos.add(ammeterDataDetailInfo1);
            // 组合无功
            LoadDataDetailInfo ammeterDataDetailInfo2 = new LoadDataDetailInfo();
            ammeterDataDetailInfo2.setCategory(AmmeterCategory.CURRENT_COMB_REACTIVE.getInfo());
            ammeterDataDetailInfo2.setTotalKwh(ammeterData.getCurrentCombReactiveTotal());
            ammeterDataDetailInfo2.setPeakKwh(ammeterData.getCurrentCombReactivePeak());
            ammeterDataDetailInfo2.setHighKwh(ammeterData.getCurrentCombReactiveHigh());
            ammeterDataDetailInfo2.setFlatKwh(ammeterData.getCurrentCombReactiveFlat());
            ammeterDataDetailInfo2.setValleyKwh(ammeterData.getCurrentCombReactiveValley());
            loadDataDetailInfos.add(ammeterDataDetailInfo2);

            ammeterDataVo.setLoadDataDetailInfo(loadDataDetailInfos);
        }

    }

}
